import gradio
import requests
from ob_tuner.functions.button_logic import *
from ob_tuner.functions.client_functions import *
from ob_tuner.util import *

# with gr.Blocks(theme="JohnSmith9982/small_and_pretty", analytics_enabled=False, ) as main_block:
with gr.Blocks(analytics_enabled=False, theme='freddyaboulton/dracula_revamped') as main_block:
    # m = gr.Markdown(f"Welcome to Gradio! {greet(Request)}")
    # gr.Button("Logout", link="/logout")
    username = gr.Textbox(label="username", visible=False)

    session_state = gr.State(value={
        "session_id": "",
        "session": requests.Session(),
        "agent": None,
        "last_response": None,
        "username": "",
        "events": [],
        "audio_warning_triggered": False,
        "message_count": 0,
        "events_initialized": False,
    })

    session_apikey = gr.State(value="")

    with gr.Accordion("My Resources") as submit_accordion:
        with gr.Tab("Selected Agent Configuration") as submit_tab:
            with gr.Row() as submit_row:
                with gr.Column(scale=3) as key_text_column:
                    with gr.Row() as key_text_row:
                        # available_client_ids = get_registered_client_ids()
                        client_id = gr.Dropdown(
                            label="Client ID",
                            filterable=True,
                            info="Develop your own AI agents or use a community agent.",
                            choices=[DEFAULT_CLIENT_ID],
                            value=DEFAULT_CLIENT_ID,
                            elem_id="client_id",
                            elem_classes=["agent_config"],
                        )
                        # initial_profiles = get_available_profile_names(client_id.value)
                        # initial_profile_names = [profile["profile_name"] for profile in initial_profiles]

                        profile_name = gr.Dropdown(
                            allow_custom_value=True,
                            filterable=True,
                            label="Profile Name",
                            info="The name of your AgentConfig. Defaults to 'default'",
                            choices=[],
                            value=DEFAULT_PROFILE_NAME,
                            elem_id="profile_name",
                            elem_classes=["agent_config"],
                        )

                with gr.Column(scale=1) as submit_column:
                    load_agent_profile_button = gr.Button(value="Reload Agent", variant="primary", elem_id="load_agent_config_button", elem_classes=["agent_config"])
                    save_agent_profile_button = gr.Button(value="Save Agent", variant="secondary", elem_id="save_agent_config_button", elem_classes=["agent_config"])

        with gr.Tab("User Profile") as user_profile_tab:
            with gr.Column():
                gr.Markdown("Information about you. When the AI agent uses tools, it will do so as you. For this reason, it will need your API keys and other identifying information.")
                with gr.Accordion("Lead Momentum Integration") as leadmo_box:
                    with gr.Row():
                        with gr.Column():
                            gradio.Markdown("These values are required for tools")
                            leadmo_api_key = gr.Textbox(info="Lead Momentum API Key",
                                                        label="Lead Momentum API Key", type="password",
                                                        placeholder="REQUIRED",
                                                        elem_id="leadmo_api_key",
                                                        elem_classes=["user_profile"])
                            leadmo_location_id = gr.Textbox(info="Lead Momentum 'Location ID'",
                                                            label="Lead Momentum Location ID",
                                                            placeholder="REQUIRED",
                                                            elem_id="leadmo_location_id",
                                                            elem_classes=["user_profile"])
                            user_leadmo_details = [leadmo_api_key, leadmo_location_id]
                            gr.Markdown(f"[API Key]({PORTAL_URL}/dashboard)")

                        with gr.Column():
                            gradio.Markdown("Optional values, used for testing", label="Optional Information")
                            leadmo_calendar_id = gr.Textbox(info="*Required", label="Lead Momentum Calendar ID", placeholder="OPTIONAL", elem_id="leadmo_calendar_id", elem_classes=["user_profile"])
                            first_name = gr.Textbox(info="Optional", label="First Name", placeholder="John", elem_id="first_name", elem_classes=["user_profile"])

                            last_name = gr.Textbox(info="Optional", label="Last Name", placeholder="Smith", elem_id="last_name", elem_classes=["user_profile"])
                            date_of_birth = gr.Textbox(info="Optional", label="Date of Birth", placeholder="1970-04-01", elem_id="date_of_birth", elem_classes=["user_profile"])
                            phone = gr.Textbox(info="*Required", label="Phone", placeholder="+16198675309", elem_id="phone", elem_classes=["user_profile"])
                            address1 = gr.Textbox(info="Optional", label="Address 1", placeholder="123 4th St.", elem_id="address1", elem_classes=["user_profile"])
                        with gr.Column():
                            gradio.Markdown("Optional values, used for testing", label="Optional Information")

                            address2 = gr.Textbox(info="Optional", label="Address 2", placeholder="Apt 123", elem_id="address2", elem_classes=["user_profile"])

                            city = gr.Textbox(info="Optional", label="City", placeholder="San Diego", elem_id="city", elem_classes=["user_profile"])
                            state = gr.Textbox(info="Optional", label="State", placeholder="CA", elem_id="state", elem_classes=["user_profile"])
                            country = gr.Textbox(info="*Required", label="Country", placeholder="US", elem_id="country", elem_classes=["user_profile"])
                            postal_code = gr.Textbox(info="Optional Code", label="Postal Code", placeholder="92108", elem_id="postal_code", elem_classes=["user_profile"])
                            website = gr.Textbox(info="Optional", label="Website", placeholder="openbra.in", elem_id="website", elem_classes=["user_profile"])

                            user_personal_details = [leadmo_calendar_id, first_name, last_name, date_of_birth, phone, address1, address2, city, state, country, postal_code, website]

                    with gr.Accordion("Landline Scrubber Integration", render=False) as lls_box:
                        lls_api_key = gr.Textbox(info="Landline Scrubber API Key", label="Landline Scrubber API Key", type="password", placeholder="YourLeadMoAPIKey", elem_id="lls_api_key", elem_classes=["user_profile"])
            with gr.Row():
                user_profile_save_button = gr.Button("Save", size="sm", variant="primary", elem_id="save_user_profile_button", elem_classes=["user_profile"])
                user_profile_load_button = gr.Button("reload", size="sm", variant="secondary", elem_id="load_user_profile_button", elem_classes=["user_profile"])

            user_profile_tab.select(load_client_details, inputs=[username],
                                           outputs=user_leadmo_details + user_personal_details)

        with gr.Tab("Stats for Nerds") as stats_tab:
            with gr.Column():
                with gr.Row():
                    with gr.Column():
                        agent_stats = gr.Image(label="Agents", elem_id="agent_stats", type="filepath", interactive=False)
                    with gr.Column():
                        review_stats = gr.Image(label="Reviews", elem_id="review_stats", type="filepath", interactive=False)

                    with gr.Column():
                        tool_stats = gr.Image(label="Tools", elem_id="tool_stats", type="filepath", interactive=False)


                    stats_tab.select(get_agent_stats, inputs=[username], outputs=[agent_stats])

                    stats_tab.select(get_review_stats, inputs=[username], outputs=[review_stats])

                    stats_tab.select(get_tool_stats, inputs=[username], outputs=[tool_stats])


    with gr.Accordion("AI Agent Configuration and Tuning", elem_classes="accordion", visible=is_settings_set(), open=False) as prompts_box:

        with gr.Tab("LLM Parameters") as tuning_tab:
            gr.Markdown(
                "Changes to these settings are used to set up a conversation using the Reset button and will not "
                "be reflected until the next 'Reset'"
            )

            with gr.Row() as preferences_row1:
                options = model_agent_config.EXECUTOR_MODEL_TYPES
                default_llm_types = ["function"]
                llm_types = gr.CheckboxGroup(
                    choices=options,
                    label="LLM Types",
                    info="List only the types of agents you are interested in.",
                    value=default_llm_types,
                    elem_id="llm_types",
                    elem_classes=["agent_config"],
                )
                original_choices = model_agent_config.FUNCTION_LANGUAGE_MODELS
                # original_value = openbrain.orm.model_agent_config.FUNCTION_LANGUAGE_MODELS[0]
                llm = gr.Dropdown(
                    choices=original_choices,
                    filterable=True,
                    # value=original_value,
                    label="LLM",
                    info="The language model to use for completion",
                    elem_id="llm",
                    elem_classes=["agent_config"],
                )

                with gr.Column() as extra_options_column:
                    record_tool_actions = gr.Checkbox(
                        label="Record Tool Actions", info="Record tool actions (use 'Actions' box).", elem_id="record_tool_actions", elem_classes=["agent_config"]
                    )
                    record_conversations = gr.Checkbox(label="Record Conversations", info="Record conversations.", elem_id="record_conversations", elem_classes=["agent_config"])

                # executor_completion_model = gr.Dropdown( choices=["text-davinci-003", "text-davinci-002",
                # "text-curie-001", "text-babbage-001", "text-ada-001"], label="Executor Completion Model",
                # info="This is the model used when Executor Model Type is set to 'completion'" )

            with gr.Row() as preferences_row2:
                executor_temp = gr.Slider(
                    minimum=0,
                    maximum=2,
                    label="Temperature",
                    step=0.1,
                    info="Higher temperature for 'spicy' agents.",
                    elem_id="executor_temp",
                    elem_classes=["agent_config"],
                )
                max_execution_time = gr.Slider(
                    minimum=0,
                    maximum=120,
                    label="Max Execution Time",
                    step=0.5,
                    info="Maximum agent response time before termination.",
                    elem_id="max_execution_time",
                    elem_classes=["agent_config"],
                )
                max_iterations = gr.Number(
                    label="Max Iterations",
                    info="Number of steps an agent can take for a response before termination.",
                    elem_id="max_iterations",
                    elem_classes=["agent_config"],
                )
        info_message = "Select tools to enable for the agent" #if LEADMO_INTEGRATION else "Select tools to enable for the agent. Lead Momentum integration is currently disabled in this environment."
        with gr.Tab("Tools") as preferences_row3:
            tools = gr.CheckboxGroup(choices=TOOL_NAMES, value=[], label="Tools", info=info_message, elem_id="tools", elem_classes=["agent_config"])

            tools.change(warn_if_missing_tool_info, inputs=[username, tools])

            gr.Markdown(
                value="Tool descriptions as presented to the AI. Confusing text here could lead to inconsistent use of tools."
            )
            _tools = discovered_tools
            with gr.Column() as tool_accordion:
                for _tool in _tools:
                    with gr.Tab(_tool.name):
                        gr.Markdown(value=get_tool_description(_tool.name), elem_id=f"{_tool.name}_description", elem_classes=["tool_description"])

        with gr.Tab("System Message") as long_text_row1:
            system_message = gr.TextArea(
                lines=10,
                label="System Message",
                placeholder="Enter your system message here",
                info="The System Message. This message is a part of every context with "
                "ChatGPT, and is therefore the most influential, and expensive place to "
                "add text",
                show_label=False,
                elem_id="system_message",
                elem_classes=["agent_config"],
            )

        with gr.Tab("Icebreaker") as long_text_row2:
            icebreaker = gr.TextArea(
                lines=10,
                label="Icebreaker",
                placeholder="Enter your icebreaker here",
                show_label=False,
                info="The first message to be sent to the user.",
                elem_id="icebreaker",
                elem_classes=["agent_config"],
            )

        with gr.Tab("Audio") as audio_tab:
            audio = gr.Audio()

        with gr.Tab("Documentation") as help_tab:
            gr.Markdown(value=get_help_text())

        with gr.Tab("Conversation Analysis") as conversation_analysis_tab:
            gr.Markdown(value="NOT IMPLEMENTED")

    with gr.Accordion("Interact with Agent") as chat_accordian:
        with gr.Row() as chat_row:
            with gr.Column(scale=2) as chat_container:
                with gr.Column(scale=2) as chat_column:
                    current_dir = os.path.dirname(os.path.abspath(__file__))
                    resources_path = os.path.join(current_dir, "resources")
                    avatars_path = os.path.join(resources_path, "avatars")
                    ai_avatar_path = os.path.join(avatars_path, "icons8-ai-64.png")
                    # human_avatar_path = os.path.join(avatars_path, "icons8-human-64.png")
                    human_avatar_path = os.path.join(avatars_path, "emotional-intelligence.png")
                    chatbot = gr.Chatbot(
                        show_share_button=False,
                        show_copy_button=True,
                        likeable=True,
                        avatar_images=(human_avatar_path, ai_avatar_path),
                        layout="bubble",
                        elem_id="chatbot",
                        elem_classes=["chatbot"],
                    )
                    with gradio.Row():
                        msg = gr.Textbox(placeholder="Type a message...", label="Message", elem_id="msg", elem_classes=["chatbot"], show_label=False, scale=20)

            with gr.Column(scale=1) as context_container:

                with gr.Row():

                    always_update_leadmo_contact = gr.Checkbox(value=True, visible=False, interactive=False)

                    update_leadmo_contact = gr.Checkbox(label="Track Contact", info="Auto create/refresh user", show_label=True, elem_id="update_leadmo_contact", elem_classes=["context"], value=False)
                    voice_enabled = gr.Checkbox(label="Speak", info="Incoming messages are spoken", show_label=True, value=False)
                fetch_user_from_leadmo_button = gr.Button("Fetch or create Leadmo user", size="sm", variant="secondary", elem_id="fetch_user_button", elem_classes=["context"], scale=10)

                context = gr.JSON(
                    label="Context",
                    # info="Additional context for tools",
                    show_label=False,
                    # show_copy_button=True,
                    # lines=4,
                    value={},
                    # elem_id="context",
                    # elem_classes=["context"],
                )

                chat_button = gr.Button("Chat", variant="primary", elem_id="chat_button", elem_classes=["chatbot"], interactive=False)
                reset_agent = gr.Button("Reset", variant="secondary", elem_id="reset_button", elem_classes=["chatbot"])


    with gr.Accordion("Debugging", open=False) as debug_box:
        with gr.Tab("Bug reports and feature requests") as feedback_tab:
            feedback_input = gr.MultimodalTextbox(
                label="Feedback",
                placeholder="Report a bug, or request a feature, attach screenshots or other helpful files.",
                show_label=False,
                lines=5,
                file_count="single",
                elem_id="feedback",
                elem_classes=["feedback"],
                file_types=["image", "video", "text"]
            )

        with gr.Tab("Gradio logs") as debug_tab:
            debug_text = gr.Textbox(
                label="Debug",
                info="Debugging information",
                show_label=False,
                lines=20,
                value=get_debug_text,
                interactive=False,
                autoscroll=True,
                show_copy_button=True,
                every=1.0,
                elem_id="debug_text",
                elem_classes=["debug"],
            )

        with gr.Tab("API Logs") as agent_debug_tab:
            api_debug_text = gr.TextArea(
                label="Debug",
                # info="Debugging information",
                show_label=False,
                lines=20,
                value=get_aws_cloudwatch_logs,
                interactive=False,
                autoscroll=True,
                show_copy_button=True,
                elem_id="api_debug_text",
                elem_classes=["debug"],
            )
            refresh_api_logs_button = gr.Button("Refresh", size="sm", variant="secondary", elem_id="refresh_api_logs_button", elem_classes=["debug"])

        with gr.Tab("Agent Logs") as agent_logs:
            agent_debug_text = gr.Textbox(
                value=get_gpt_agent_logs,
                label="Debug",
                info="GptAgent logs",
                show_label=False,
                lines=20,
                interactive=False,
                autoscroll=True,
                show_copy_button=True,
                every=3.0,
                elem_id="agent_debug_text",
                elem_classes=["debug"],
            )
            refresh_agent_logs_button = gr.Button("Refresh", size="sm", variant="secondary")

        with gr.Tab("Action Event Logs"):
            # events_str = get_action_events()
            # events = gr.Json(value=events_str, label="Latest action event recorded.")
            events = gr.Json(
                value={},
                # every=15.0,
                label="Latest action recorded.",
                elem_id="events",
                elem_classes=["debug"],
            )


    with gr.Row() as bottom_text_row:
        logout_button = gr.Button("Logout", link="/logout", variant="secondary", size="sm", scale=1, elem_id="logout_button", elem_classes=["auth"])

        with gr.Column(scale=20):
            bottom_text = gr.Markdown(value=get_bottom_text(), rtl=True, elem_id="bottom_text", elem_classes=["bottom_text"])

    preferences = [
        icebreaker,
        system_message,
        llm,
        max_iterations,
        max_execution_time,
        executor_temp,
        profile_name,
        client_id,
        record_tool_actions,
        record_conversations,
        tools,
        llm_types,
    ]
    fetch_user_from_leadmo_button.click(fetch_user_from_leadmo,
                                        inputs=[always_update_leadmo_contact, context, username],
                                        outputs=[context])

    user_profile_save_button.click(fill_context, inputs=[username] + user_leadmo_details + user_personal_details,
                                   outputs=[context])
    for context_item in user_personal_details + user_leadmo_details:
        context_item.change(fill_context, inputs=[username] + user_leadmo_details + user_personal_details, outputs=[context])

    # Refresh Buttons
    refresh_api_logs_button.click(get_aws_cloudwatch_logs, inputs=[session_state], outputs=[api_debug_text])
    refresh_agent_logs_button.click(get_gpt_agent_logs, outputs=[agent_debug_text])
    # refresh_events_button.click(get_action_events, inputs=[events, session_state], outputs=[events])
    user_profile_load_button.click(load_client_details, inputs=[username], outputs=user_leadmo_details + user_personal_details)

    # Save Buttons
    user_profile_save_button.click(save_client_details, inputs=[username] + user_leadmo_details + user_personal_details, outputs=[])


    # Save Agent Button
    save_agent_profile_button.click(
        save,
        inputs=[
            icebreaker,
            system_message,
            llm,
            max_iterations,
            max_execution_time,
            executor_temp,
            profile_name,
            client_id,
            record_tool_actions,
            record_conversations,
            tools,
        ],
    )
    # main_block.load(update_client_id, [username, session_state], [client_id])
    # Load Agent Button
    load_agent_profile_button.click(load, inputs=[profile_name, client_id], outputs=preferences)

    # Chat Button
    chat_button.click(fn=chat, inputs=[msg, chatbot, profile_name, session_state, client_id, context], outputs=[msg, chatbot, session_state, context]).then(alert_on_actions, inputs=[session_state], outputs=[session_state, events]).then(fetch_user_from_leadmo, inputs=[update_leadmo_contact, context, username], outputs=[context])
    chat_button.click(get_aws_cloudwatch_logs, inputs=[session_state], outputs=[agent_debug_text])
    chat_button.click(get_bottom_text, inputs=[session_state, client_id, profile_name], outputs=[bottom_text])

    msg.submit(chat, [msg, chatbot, profile_name, session_state, client_id, context], [msg, chatbot, session_state, context]).then(alert_on_actions, inputs=[session_state], outputs=[session_state, events]).then(fetch_user_from_leadmo, inputs=[update_leadmo_contact, context, username], outputs=[context])
    msg.submit(get_aws_cloudwatch_logs, inputs=[session_state], outputs=[agent_debug_text])
    msg.submit(get_bottom_text, inputs=[session_state, client_id, profile_name], outputs=[bottom_text])

    # Chat Reset Button
    reset_agent.click(
        fn=reset,
        inputs=[client_id, profile_name, chatbot, session_state, context],
        outputs=[msg, chatbot, session_state, context],
    )#.then(alert_on_actions, inputs=[session_state], outputs=[session_state, events])
    reset_agent.click(get_bottom_text, inputs=[session_state, client_id, profile_name], outputs=[bottom_text])
    # reset_agent.click(get_bottom_text, inputs=[session_state, profile_name, client_id], outputs=[context])
    reset_agent.click(get_aws_cloudwatch_logs, inputs=[session_state], outputs=[agent_debug_text])
    # reset_agent.click(get_action_events, inputs=[events, session_state], outputs=[events])
    reset_agent.click(enable_chat_button, outputs=[chat_button])
    feedback_input.submit(submit_feedback, inputs=[feedback_input, username], outputs=[])
    reset_agent.click(alert_on_actions, inputs=[session_state], outputs=[session_state, events]).then(fetch_user_from_leadmo, inputs=[update_leadmo_contact, context, username], outputs=[context])

    # On Change Events
    llm_types.change(get_llm_choices, inputs=[llm_types], outputs=[llm])
    client_id.change(update_available_profile_names, inputs=[client_id], outputs=[profile_name])
    client_id.change(enable_or_disable_save, inputs=[client_id, profile_name], outputs=[save_agent_profile_button])
    profile_name.change(enable_or_disable_save, inputs=[client_id, profile_name], outputs=[save_agent_profile_button])

    # profile_name.change(load, inputs=[profile_name, client_id], outputs=preferences)
    main_block.load(initialize_username, [], [username, client_id])

    username.change(load, inputs=[profile_name, username], outputs=preferences)
    username.change(set_client_id, inputs=[username], outputs=[client_id])
    # username.change(generate_context, [username], [context])
    username.change(fill_context_from_username, inputs=[username], outputs=[context])

    chatbot.change(speak, inputs=[voice_enabled, chatbot, session_state], outputs=[audio])
    chatbot.like(react_to_message, [chatbot, session_state, profile_name, client_id], [])


if __name__ == "__main__":
    if os.getenv("GRADIO_PORT"):
        gradio_host = os.getenv("GRADIO_HOST", '0.0.0.0')
        gradio_port = int(os.getenv("GRADIO_PORT", 7861))
        main_block.launch(
            debug=True,
            share=False,
            server_name=gradio_host,
            server_port=gradio_port,
        )



import uvicorn
import jwt
from fastapi import Request, FastAPI, Depends
from starlette.middleware.sessions import SessionMiddleware
from starlette.responses import RedirectResponse, JSONResponse, FileResponse
from ob_tuner.main_page import main_block as io
from ob_tuner.landing_page import landing_page
from ob_tuner.util import *

CUSTOM_PATH = "/gradio"
app = FastAPI()
app.add_middleware(SessionMiddleware, secret_key=SUPER_SECRET_COGNITO_KEY)



@app.get("/health")
def health():
    return {"status": "ok"}


@app.get("/login")
def login():
    # Redirect to the Cognito Hosted UI

    cognito_login_url = (
        f"{COGNITO_DOMAIN}/login?client_id={CLIENT_ID}" f"&response_type=code&scope=email+openid&redirect_uri={DEPLOYMENT_URL}/callback"
    )
    # logger.debug(f"Redirecting to: {cognito_login_url}")
    return RedirectResponse(cognito_login_url)


@app.get("/callback")
def callback(request: Request, code: str):
    token_url = f"{COGNITO_DOMAIN}/oauth2/token"
#     logger.debug(f"Back from callback- token URL: {token_url}")
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    data = {
        "grant_type": "authorization_code",
        "client_id": CLIENT_ID,
        "client_secret": CLIENT_SECRET,
        "code": code,
        "redirect_uri": DEPLOYMENT_URL + '/callback',
    }
#     logger.debug(f"Data: {data}")
    try:
        response = requests.post(token_url, headers=headers, data=data)
    except requests.exceptions.RequestException as e:
#         logger.error(f"Could not get token from {COGNITO_DOMAIN}/oauth2/token")
        return JSONResponse(status_code=500, content={"message": f"Authentication failed - can't find {COGNITO_DOMAIN}/oauth2/token"})
    except Exception as e:
#         logger.error(f"Error getting token: {e}")
        return JSONResponse(status_code=500, content={"message": "Authentication failed WTF"})

    if response.status_code == 200:
        url = "/gradio/"
        tokens = response.json()
        id_token = tokens.get("id_token")
        response = RedirectResponse(url)

        response.set_cookie(key="id_token", value=id_token, httponly=True, secure=True)
        return response
    else:
        # Handle error response
        # add data headers and token_url to the response for troubleshooting

        response_message = {
            "message": f"Authentication failed - {response.status_code}",
            "data": data,
            "headers": headers,
            "token_url": token_url,
        }
#         logger.error(response_message)
        logger.debug(f"Trying to redirect to HTTPS: {DEPLOYMENT_URL}/login-demo")
        return RedirectResponse(url=f'{DEPLOYMENT_URL}/login-demo')

        # return JSONResponse(status_code=response.status_code, content={"message": "Authentication failed"})


def ensure_user_exists(decoded_token: dict, disable=True):
    if disable:
        return
    groups = decoded_token.get("cognito:groups")
    email_verified = decoded_token.get("email_verified")
    preferred_role = decoded_token.get("cognito:preferred_role")
    username = decoded_token.get("cognito:username")
    roles = decoded_token.get("cognito:roles")
    token_use = decoded_token.get("token_use")
    email = decoded_token.get("email")

    try:
        user = Client.get(email=email)
        # update the user
        user.groups = groups
        user.email_verified = email_verified
        user.preferred_role = preferred_role
        user.username = username
        user.roles = roles
        user.token_use = token_use
    except Exception as e:
        user = Client(
            email=email,
            groups=groups,
            email_verified=email_verified,
            preferred_role=preferred_role,
            username=username,
            roles=roles,
            token_use=token_use,
            location_id="UNKNOWN",
            lls_api_key=None,
            leadmo_api_key=None,
        )
    user.save()


def get_user(request: Request) -> str or None:
    try:
        id_token = request.cookies["id_token"]
    except KeyError:
        return None

    try:
        decoded_token = jwt.decode(id_token, options={"verify_signature": False}, algorithms=["RS256"])
        ensure_user_exists(decoded_token)
        email = decoded_token.get("email")
        return email
    except jwt.PyJWTError as e:
        print(f"Token decoding error: {e}")
        return None


@app.get('/')
def public(user: dict = Depends(get_user)):
    if user:
        logger.debug(f"Trying to redirect to HTTPS: {DEPLOYMENT_URL}/gradio")
        return RedirectResponse(url=f'{DEPLOYMENT_URL}/gradio')
        # return RedirectResponse(url=f'/gradio')
    else:
        logger.debug(f"Trying to redirect to HTTPS: {DEPLOYMENT_URL}/login-demo")
        return RedirectResponse(url=f'{DEPLOYMENT_URL}/login-demo')
        # return RedirectResponse(url=f'/login-demo')

@app.route('/signup')
async def signup(request: Request):
    signup_url = f"{COGNITO_DOMAIN}/signup?client_id={CLIENT_ID}" f"&response_type=code&scope=email+openid&redirect_uri={DEPLOYMENT_URL}/callback"
    return RedirectResponse(signup_url)


@app.route('/logout')
async def logout(request: Request):
    # logout_url = f"{COGNITO_DOMAIN}/logout?response_type=token&client_id={CLIENT_ID}&redirect_uri={CALLBACK_URL}"
    # logout_url = f"{COGNITO_DOMAIN}/logout?client_id={CLIENT_ID}&logout_uri={CALLBACK_URL}
    try:
        request.cookies.clear()
    except KeyError:
        pass

    logout_url = f"{COGNITO_DOMAIN}/logout?client_id={CLIENT_ID}&logout_uri={DEPLOYMENT_URL}/login-demo"
    return RedirectResponse(logout_url)

    # request.session.pop('user', None)
    # return RedirectResponse(url='/')
@app.get('/favicon.ico', include_in_schema=False)
async def favicon():
    current_file_path = os.path.dirname(os.path.abspath(__file__))
    favicon_path = os.path.join(current_file_path, 'resources/favicon/favicon.ico')
    return FileResponse(favicon_path)


app = gr.mount_gradio_app(app, landing_page, path="/login-demo")

# app  = gr.mount_gradio_app(app, landing_page, path='/')

app = gr.mount_gradio_app(app, io, path=CUSTOM_PATH, auth_dependency=get_user)


if __name__ == "__main__":
    if os.getenv("GRADIO_PORT", False) or os.getenv("GRADIO_HOST", False):
        gradio_host = os.getenv("GRADIO_HOST", '0.0.0.0')
        gradio_port = int(os.getenv("GRADIO_PORT", 8000))
        uvicorn.run(app, host=gradio_host, port=gradio_port)
    else:
        uvicorn.run(app, host="0.0.0.0", port=80)



import requests

from ob_tuner.functions.button_logic import *
from ob_tuner.util import *

WELCOME_MESSAGE = f"""
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>OpenBra.in Demo</title>
    <link rel="stylesheet" href="styles.css">
</head>
<body>
    <header>
        <h1>Welcome to OpenBra.in</h1>
        <p>OpenBra.in is a comprehensive platform designed to streamline the process of creating, testing, and deploying AI agents...</p>
    </header>

    <section id="features">
        <h2>Features</h2>
        <ul>
            <li>Interactive GUI: User-friendly Gradio interface for easy configuration and testing of AI agents.</li>
            <li>Real-time Testing: Immediate feedback on agent configurations and tool use.</li>
            <li>Flexible Configuration: Save, load, and manage various agent configurations.</li>
            <li>AWS Integration: Scalable, reliable infrastructure.</li>
            <li>Event-Driven Architecture: Efficient handling of actions and information retrieval through an event bus.</li>
            <li>Comprehensive Toolset: Includes tools for CRM integration, time management, and more.</li>
        </ul>
    </section>

    <section id="how-it-works">
        <h2>How It Works</h2>
        <p>OpenBra.in is built on a robust technical foundation that includes...</p>
        
        <h3>AI Tuning App</h3>
        <p>OB-Tuner is a user-friendly interface built on Gradio for creating, testing, and managing AI agent configurations...</p>
        
        <h3>OpenBrain API</h3>
        <p>The OpenBrain API provides a single chat endpoint for interacting with AI agents. Provide your `api_key`, your `client_id`, and the name of your saved agent.</p>
        
        <h3>Event-Driven Architecture</h3>
        <p>OpenBra.in uses an event bus to manage actions and information retrieval. This allows for recovery of failed tool runs using a DLQ.</p>
        
        <h3>Toolset</h3>
        <p>OpenBra.in includes a comprehensive set of tools for CRM integration with Lead Momentum. Tools can be enabled or disabled as needed on a per-agent basis.</p>
        
        <h3>RAG Infrastructure</h3>
        <p>Record agent tool use, and enable recording on a per agent basis. Use the `like` and `dislike` buttons to help fill the vector store with good example responses.</p>
                
    </section>

    <section id="getting-started">
        <h2>Getting Started</h2>
        <p>Steps to start using OpenBra.in...</p>
        <p>Installation and setup instructions...</p>
    </section>

    <section id="contact-support">
        <h2>Visit us on GitHub</h2>
        <a href="https://github.com/svange/openbrain/">OpenBra.in on GitHub</a>
    </section>

    <footer>
        <p>&copy; 2024 OpenBra.in</p>
    </footer>
</body>
</html>
"""

with gr.Blocks(analytics_enabled=False, theme="freddyaboulton/dracula_revamped") as landing_page:
    # Read the README file content
    with gr.Row():
        with gr.Column(scale=15):
            if NOAUTH_DEMO_PAGE:
                gr.Markdown(
                    value="# Welcome to OpenBra.in!\n Please enjoy the live demo. Load an agent profile, and start chatting with the AI. Explore the tools. If you are interested in trying OpenBra.in, please sign up for an account. Once signed up you can create and save your own agent profiles.\nDeployed agents can be used in your own applications via the OpenBra.in API, the following website is using the `openbrain` `promoter` agent.\n\n[OpenBra.in Demo Website](https://www.openbra.in/) (allow 15 seconds for the agent to initialize).")
            else:
                gr.Markdown(value="# Welcome!\n Please sign up or login. ")

        with gr.Column(scale=1) as login_column:
            gr.Button("Sign up / Login", link="/login", variant="secondary", size="sm")
            # gr.Button("Signup", link="/signup", variant="secondary", size="sm")

    if NOAUTH_DEMO_PAGE:

        with gr.Tab("Demo"):
            session_state = gr.State(value={
                "session_id": "",
                "session": requests.Session(),
                "agent": None,
                "last_response": None,
                "username": "",
                "events": [],
                "audio_warning_triggered": False,
                # "initial_reset": False
            })

            with gr.Tab("Selected Agent Configuration") as submit_tab:
                with gr.Row() as submit_row:
                    with gr.Column(scale=3) as key_text_column:
                        with gr.Row() as key_text_row:
                            client_id = gr.Dropdown(
                                label="Client ID",
                                info="Develop your own AI agents or use a community agent.",
                                choices=["openbrain"],
                                value="openbrain",
                            )

                            profile_name = gr.Dropdown(
                                allow_custom_value=True,
                                label="Profile Name",
                                info="The name of your AgentConfig. Defaults to 'default'",
                                choices=[str(DEFAULT_PROFILE_NAME), 'promoter'],
                                value='promoter',
                                interactive=False,
                            )

                    with gr.Column(scale=1) as submit_column:
                        load_agent_profile_button = gr.Button(value="Reload Agent", variant="secondary")
                        save_agent_profile_button = gr.Button(value="Save Agent", variant="secondary", render=True, interactive=False)

            with gr.Accordion("AI Agent Configuration and Tuning", elem_classes="accordion", visible=is_settings_set(),
                              open=False) as prompts_box:
                with gr.Tab("LLM Parameters") as tuning_tab:
                    gr.Markdown(
                        "Changes to these settings are used to set up a conversation using the Reset button and will not "
                        "be reflected until the next 'Reset'"
                    )

                    with gr.Row() as preferences_row1:
                        options = model_agent_config.EXECUTOR_MODEL_TYPES
                        default_llm_types = ["function"]
                        llm_types = gr.CheckboxGroup(
                            choices=options,
                            label="LLM Types",
                            info="List only the types of agents you are interested in.",
                            value=default_llm_types,
                        )
                        original_choices = model_agent_config.FUNCTION_LANGUAGE_MODELS
                        # original_value = openbrain.orm.model_agent_config.FUNCTION_LANGUAGE_MODELS[0]
                        llm = gr.Dropdown(
                            choices=original_choices,
                            # value=original_value,
                            label="LLM",
                            info="The language model to use for completion",
                        )

                        with gr.Column() as extra_options_column:
                            record_tool_actions = gr.Checkbox(
                                label="Record Tool Actions", info="Record tool actions (use 'Actions' box).",
                                interactive=False
                            )
                            record_conversations = gr.Checkbox(label="Record Conversations",
                                                               info="Record conversations.",
                                                               interactive=False)

                        # executor_completion_model = gr.Dropdown( choices=["text-davinci-003", "text-davinci-002",
                        # "text-curie-001", "text-babbage-001", "text-ada-001"], label="Executor Completion Model",
                        # info="This is the model used when Executor Model Type is set to 'completion'" )

                    with gr.Row() as preferences_row2:
                        executor_temp = gr.Slider(
                            minimum=0,
                            maximum=2,
                            label="Temperature",
                            step=0.1,
                            info="Higher temperature for 'spicy' agents.",
                        )
                        max_execution_time = gr.Slider(
                            minimum=0,
                            maximum=120,
                            label="Max Execution Time",
                            step=0.5,
                            info="Maximum agent response time before termination.",
                        )
                        max_iterations = gr.Number(
                            label="Max Iterations",
                            info="Number of steps an agent can take for a response before termination.",
                        )

                with gr.Tab("Tools") as preferences_row3:
                    tools = gr.CheckboxGroup(choices=TOOL_NAMES, value=[], label="Tools",
                                             info="Select tools to enable for the agent", interactive=False)

                    gr.Markdown(
                        value="Tool descriptions as presented to the AI. Confusing text here could lead to inconsistent use of tools."
                    )
                    _tools = discovered_tools
                    with gr.Column() as tool_accordion:
                        for _tool in _tools:
                            with gr.Tab(_tool.name):
                                gr.Markdown(value=get_tool_description(_tool.name))

                with gr.Tab("System Message") as long_text_row1:
                    system_message = gr.TextArea(
                        lines=10,
                        label="System Message",
                        placeholder="Enter your system message here",
                        info="The System Message. This message is a part of every context with "
                             "ChatGPT, and is therefore the most influential, and expensive place to "
                             "add text",
                        show_label=False,
                    )

                with gr.Tab("Icebreaker") as long_text_row2:
                    icebreaker = gr.TextArea(
                        lines=10,
                        label="Icebreaker",
                        placeholder="Enter your icebreaker here",
                        show_label=False,
                        info="The first message to be sent to the user.",
                    )

                with gr.Tab("Audio") as audio_tab:
                    audio = gr.Audio()

                with gr.Tab("Documentation") as help_tab:
                    gr.Markdown(value=get_help_text())

            with gr.Accordion("Interact with Agent") as chat_accordian:
                with gr.Row() as chat_row:
                    with gr.Column(scale=2) as chat_container:
                        with gr.Column(scale=2) as chat_column:
                            chatbot = gr.Chatbot(
                                show_share_button=True,
                                show_copy_button=True,
                                # avatar=(user_avatar_path, ai_avatar_path),
                                likeable=True,
                                # layout="panel",
                                layout="bubble",

                            )
                            with gr.Row():
                                msg = gr.Textbox(placeholder="Type a message...", label="Message", elem_id="msg",
                                                 elem_classes=["chatbot"], show_label=False, scale=20)
                                voice_enabled = gr.Checkbox(label="", scale=1, info="Speak", show_label=False )

                    with gr.Column(scale=1) as context_container:
                        initial_value_str = json.loads(json.dumps({
                            "first_name": "John",
                            "last_name": "Doe",
                            "locationId": "loc-123456",
                            "calendarId": "cal-123456",
                            "random_word": "Rambutan"
                        }))

                        gr.Markdown("OpenBra.in's API accepts arbitrary key/value pairs. Tools like book appointment require parameters like `calendarId=23nio2n3nf2oi32`, which cause halucinations. This gets around that by creating tools with values sent in by the API call.")
                        context = gr.JSON(
                            label="Context",
                            value=initial_value_str
                        )
                        landing_page.load(lambda: gr.JSON(label="Context", value=initial_value_str), outputs=[context])

                        chat_button = gr.Button("Chat", variant="primary", interactive=False)
                        reset_agent = gr.Button("Reset", variant="secondary")

                        msg.submit(
                            chat,
                            [msg, chatbot, profile_name, session_state, client_id, context],
                            [msg, chatbot, session_state, context],
                        )

            with gr.Row() as bottom_text_row:
                logout_button = gr.Button("Logout", link="/logout", variant="secondary", size="sm", scale=1, render=False)
                with gr.Column(scale=10):
                    bottom_text = gr.Markdown(value=get_bottom_text(), rtl=True)

            preferences = [
                icebreaker,
                system_message,
                llm,
                max_iterations,
                max_execution_time,
                executor_temp,
                profile_name,
                client_id,
                record_tool_actions,
                record_conversations,
                tools,
                llm_types,
            ]

            # Refresh Buttons

            save_agent_profile_button.click(
                fn=lambda: gr.Warning("Saving disabled in the demo, please sign up for an account."))
            load_agent_profile_button.click(load, inputs=[profile_name, client_id], outputs=preferences)

            profile_name.change(load, inputs=[profile_name, client_id], outputs=preferences)
            landing_page.load(load, inputs=[profile_name, client_id], outputs=preferences)

            # Chat Button
            chat_button.click(
                fn=chat,
                inputs=[msg, chatbot, profile_name, session_state, client_id, context],
                outputs=[msg, chatbot, session_state, context],
            )
            dummy_text = gr.Textbox(visible=False)
            chat_button.click(get_aws_cloudwatch_logs, inputs=[session_state], outputs=[dummy_text])

            # Chat Reset Button
            reset_agent.click(
                fn=reset,
                inputs=[client_id, profile_name, chatbot, session_state, context],
                outputs=[msg, chatbot, session_state, context],
            )
            # reset_agent.click(get_bottom_text, inputs=[session_state, profile_name, client_id], outputs=[context])
            reset_agent.click(get_aws_cloudwatch_logs, inputs=[session_state], outputs=[dummy_text])
            # reset_agent.click(get_action_events, inputs=[dummy_text, session_state], outputs=[dummy_text])
            reset_agent.click(enable_chat_button, outputs=[chat_button])

            # On Change Events
            llm_types.change(get_llm_choices, inputs=[llm_types], outputs=[llm])
            # client_id.change(update_available_profile_names, inputs=[client_id], outputs=[profile_name])
            chatbot.change(speak, inputs=[voice_enabled, chatbot, session_state], outputs=[audio])

    with gr.Tab(f"Welcome to {CUSTOMER_NAME}"):
        gr.HTML(WELCOME_MESSAGE)


if __name__ == "__main__":
    if os.getenv("GRADIO_PORT"):
        gradio_host = os.getenv("GRADIO_HOST", '0.0.0.0')
        gradio_port = int(os.getenv("GRADIO_PORT", 7861))
        landing_page.launch(
            debug=True,
            share=False,
            server_name=gradio_host,
            server_port=gradio_port,
        )



import datetime
import json
import os
import pathlib
import time
from decimal import Decimal
from logging.handlers import RotatingFileHandler
import boto3
import faker
import gradio as gr
import logging
from io import StringIO
from sys import platform

import requests
from dotenv import load_dotenv, find_dotenv
from importlib.metadata import version
from boto3.dynamodb.conditions import Attr
# Use find_dotenv to locate the .env file
dotenv_path = find_dotenv(usecwd=True)
if dotenv_path:
    load_dotenv(dotenv_path)

os.environ["INFRA_STACK_NAME"] = os.environ.get("INFRA_STACK_NAME", "LOCAL")
# Unfortunate... fix later
from openbrain.orm.model_client import Client

import openbrain.orm
import openbrain.util
from openbrain.orm.model_agent_config import AgentConfig
from openbrain.orm.model_common_base import InMemoryDb
from openbrain.util import config, Defaults
from openbrain.tools import Toolbox


class CustomJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Decimal):
            return float(obj)
        return super(CustomJsonEncoder, self).default(obj)

# Logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

# Add a rotating log handler
formatter = logging.Formatter('%(filename)-20s:%(lineno)-4d %(levelname)-8s %(message)s')

current_dir = pathlib.Path(__file__).parent
parent_dir = current_dir.parent
pathlib.Path(parent_dir / "logs").mkdir(parents=True, exist_ok=True)
pathlib.Path(parent_dir / "temp").mkdir(parents=True, exist_ok=True)
LOG_FILE_PATH = parent_dir / "logs" / "ob-tuner.log"
TEMP_DIR_PATH = parent_dir / "temp"


handler = RotatingFileHandler(LOG_FILE_PATH, maxBytes=10000, backupCount=5)
handler.setLevel(logging.DEBUG)
handler.setFormatter(formatter)
logger.addHandler(handler)

# Add a stringio handler
log_stream = StringIO()
string_handler = logging.StreamHandler(log_stream)
string_handler.setLevel(logging.DEBUG)
string_handler.setFormatter(formatter)
logger.addHandler(string_handler)
ADMIN_USER = os.environ.get("ADMIN_USER")

if os.environ.get("OB_MODE", "LOCAL") == "LOCAL" and ADMIN_USER:
    logger.info(f"Creating admin user {ADMIN_USER}")
    try:
        ADMIN_LEADMO_LOCATION_ID = os.getenv("ADMIN_LEADMO_LOCATION_ID")
        ADMIN_LEADMO_API_KEY = os.getenv("ADMIN_LEADMO_API_KEY")
        ADMIN_LEADMO_CALENDAR_ID = os.getenv("ADMIN_LEADMO_CALENDAR_ID")
        admin_user = Client(email=ADMIN_USER, leadmo_api_key=ADMIN_LEADMO_API_KEY, leadmo_location_id=ADMIN_LEADMO_LOCATION_ID, leadmo_calendar_id=ADMIN_LEADMO_CALENDAR_ID)
        admin_user.save()
        admin_default_agent_config = AgentConfig(client_id=ADMIN_USER, profile_name=Defaults.DEFAULT_PROFILE_NAME.value)
        admin_default_agent_config.save()
    except Exception as e:
        admin_user = Client(email=ADMIN_USER)
        admin_user.save()
        logger.info(f"Created admin user {ADMIN_USER}")

INFRA_STACK_NAME = os.environ.get("INFRA_STACK_NAME")
OB_MODE = config.OB_MODE
CHAT_ENDPOINT = os.environ.get("OB_API_URL", "") + "/chat"
DEFAULT_ORIGIN = os.environ.get("DEFAULT_ORIGIN", "https://localhost:5173")
OB_PROVIDER_API_KEY = os.environ.get("OB_PROVIDER_API_KEY", "")
DEFAULT_CLIENT_ID: str = openbrain.util.Defaults.DEFAULT_CLIENT_ID.value
DEFAULT_PROFILE_NAME = Defaults.DEFAULT_PROFILE_NAME.value
PORTAL_URL = os.getenv("PORTAL_URL")
COGNITO_DOMAIN = os.getenv("COGNITO_DOMAIN")
CLIENT_ID = os.getenv("CLIENT_ID")
# CALLBACK_URL = os.getenv("CALLBACK_URL")
DEPLOYMENT_URL = os.getenv("DEPLOYMENT_URL")
CLIENT_SECRET = os.getenv("CLIENT_SECRET")
GUEST_CLIENT_ID = "guest@openbra.in"
discovered_tools = Toolbox.discovered_tools
model_agent_config = openbrain.orm.model_agent_config
OB_TUNER_VERSION = version("ob-tuner")
CUSTOMER_NAME = os.getenv("CUSTOMER_NAME")
SUPER_SECRET_COGNITO_KEY = os.getenv("SUPER_SECRET_COGNITO_KEY")

NOAUTH_DEMO_PAGE_STR = os.getenv("NOAUTH_DEMO_PAGE", "False")
LEADMO_INTEGRATION_STR = os.getenv("LEADMO_INTEGRATION", "False")
LLS_INTEGRATION_STR = os.getenv("LLS_INTEGRATION", "False")
NOAUTH_DEMO_PAGE = NOAUTH_DEMO_PAGE_STR.casefold() == "true"
LEADMO_INTEGRATION = LEADMO_INTEGRATION_STR.casefold() == "true"
LLS_INTEGRATION = LLS_INTEGRATION_STR.casefold() == "true"

logger.info(f"************************************ GRADIO CONFIG ************************************")
logger.info(f"DEFAULT_ORIGIN: {DEFAULT_ORIGIN}")
logger.info("OB_API_URL: " + os.environ.get("OB_API_URL", ""))
logger.info(f"({type(NOAUTH_DEMO_PAGE)}): {NOAUTH_DEMO_PAGE=}")
logger.info(f"({type(LEADMO_INTEGRATION)}): {LEADMO_INTEGRATION=}")
logger.info(f"({type(LLS_INTEGRATION)}): {LLS_INTEGRATION=}")
logger.info(f"{CUSTOMER_NAME=}")
logger.info(f"{LEADMO_INTEGRATION=}")
logger.info(f"{NOAUTH_DEMO_PAGE=}")
logger.info(f"{LLS_INTEGRATION=}")
logger.info(f"{OB_TUNER_VERSION=}")
logger.info(f"LOGGING PATH: {LOG_FILE_PATH}")
logger.info(f"************************************ AUTH ************************************")
logger.info(f'COGNITO_DOMAIN: {COGNITO_DOMAIN}')
logger.info(f'CLIENT_ID: {CLIENT_ID}')
# logger.info(f'CALLBACK_URL: {CALLBACK_URL}')
logger.info(f'{PORTAL_URL=}')
logger.info(f'DEPLOYMENT_URL: {DEPLOYMENT_URL}')
logger.info("DEFAULT_ORIGIN: " + os.environ.get("DEFAULT_ORIGIN", "https://localhost:5173"))

logger.info(f"************************************ OPENBRAIN CONFIG ************************************")
logger.info(f"OB_MODE: {OB_MODE}")
logger.info(f"SESSION_TABLE_NAME: {config.SESSION_TABLE_NAME}")
logger.info(f"AGENT_CONFIG_TABLE_NAME: {config.AGENT_CONFIG_TABLE_NAME}")
logger.info(f"ACTION_TABLE_NAME: {config.ACTION_TABLE_NAME}")
logger.info(f"{INFRA_STACK_NAME=}")

logger.info(f"************************************ SECRETS ************************************")
OBFUSCATED_CLIENT_SECRET = CLIENT_SECRET[:2] + "-" * (len(CLIENT_SECRET) - 2)
OBFUSCATED_OB_PROVIDER_API_KEY = os.environ.get("OB_PROVIDER_API_KEY", "")[:2] + "-" * (len(os.environ.get("OB_PROVIDER_API_KEY", "")) - 2)
logger.info(f'CLIENT_SECRET: {OBFUSCATED_CLIENT_SECRET}')
logger.info("OB_PROVIDER_API_KEY: " + OBFUSCATED_OB_PROVIDER_API_KEY)

string_handler.flush()
tool_names = [tool.name for tool in Toolbox.discovered_tools if not tool.name.startswith("leadmo_")]
leadmo_tool_names = [tool.name for tool in Toolbox.discovered_tools if tool.name.startswith("leadmo_")]

tool_names.sort()
leadmo_tool_names.sort()
# TOOL_NAMES = tool_names + leadmo_tool_names if LEADMO_INTEGRATION else tool_names
TOOL_NAMES = tool_names + leadmo_tool_names
logger.info(f"Tools ({len(TOOL_NAMES)}): {TOOL_NAMES}")

REGISTERED_CLIENT_IDS = {
    "WoxomAI": [DEFAULT_CLIENT_ID, "woxom", "leadmo"],
    "OpenBra.in": [DEFAULT_CLIENT_ID, 'openbrain', 'leadmo'],
    "OBTest": [DEFAULT_CLIENT_ID, 'openbrain', 'woxom', 'leadmo'],
}


def get_debug_text(_debug_text=None) -> str:
    try:
        ret = log_stream.getvalue()
    except Exception as e:
        ret = e.__str__()

    if _debug_text:
        _debug_text = ret

    return ret


def get_aws_xray_trace_summaries(id=None):
    """Get x-ray logs from AWS"""
    client = boto3.client("xray")
    this_year = datetime.datetime.now().year
    this_month = datetime.datetime.now().month
    this_day = datetime.datetime.now().day
    yesterday = (datetime.datetime.now() - datetime.timedelta(days=1)).day

    if id:
        response = client.get_trace_summaries(
            StartTime=datetime.datetime(this_year, this_month, yesterday),
            EndTime=datetime.datetime(this_year, this_month, this_day),
            Sampling=False,
            FilterExpression=f"traceId = {id}",
        )
    else:
        response = client.get_trace_summaries(
            StartTime=datetime.datetime(this_year, this_month, yesterday),
            EndTime=datetime.datetime(this_year, this_month, this_day),
            Sampling=False,
        )

    return response


def is_settings_set():
    return True


def get_tool_description(tool_name):
    for tool in Toolbox.discovered_tools:
        if tool_name == tool.name:
            tool_instance = tool.tool()
            tool_description = tool_instance.description
            fields = tool_instance.args_schema.model_fields
            args_string = ""
            if not fields:
                args_string = "'No args'"
            for field in fields:
                field_str = fields[field].__str__()
                args_string += f"{field}: {field_str}\n"
            tool_description = f"""#### Description
{tool_description}

#### Args
```python
{args_string}
```"""
            return tool_description


def get_available_tool_descriptions():
    tool_descriptions = []

    for tool in Toolbox.discovered_tools:
        tool_name = tool.name
        tool_instance = tool.tool()
        tool_description = tool_instance.description
        fields = tool_instance.args_schema.model_fields
        args_string = ""
        if not fields:
            args_string = "'No args'"
        for field in fields:
            field_str = fields[field].__str__()
            args_string += f"{field}: {field_str}\n"
        tool_description = f"""
## Tool: {tool_name}

#### Description
{tool_description}

#### Args
```python
{args_string}
```
---"""

        tool_descriptions.append(tool_description)

    tool_descriptions_string = "\n".join(tool_descriptions)
    return tool_descriptions_string


def get_available_profile_names(_client_id) -> list:
    logger.debug(f"Getting available profile names for {_client_id}")
    # logger.warning("get_available_profile_names() is not implemented")
    # Get AgentConfig table

    if OB_MODE == Defaults.OB_MODE_LOCAL.value:
        try:
            logger.debug(f"Checking InMemoryDb for available profile names...")
            lst = list(InMemoryDb.instance[config.AGENT_CONFIG_TABLE_NAME][_client_id].keys())
            return lst
        except Exception:
            logger.warning(f"Not found, initializing default profile name...")
            _client_id = _client_id or GUEST_CLIENT_ID
            default_config = AgentConfig(client_id=_client_id, profile_name=DEFAULT_PROFILE_NAME)
            logger.info(f"Saving default profile name: {default_config}")
            default_config.save()
            logger.debug(f"Saved default profile name: {default_config}")
            lst = list(InMemoryDb.instance['agent_config_table'][_client_id].keys())
            logger.debug(f"Available profile names: {lst}")
            return lst
    else:
        table = boto3.resource("dynamodb").Table(config.AGENT_CONFIG_TABLE_NAME)
        # get all items in the table
        try:
            items = table.scan()["Items"]
            available_profile_names = [item["profile_name"] for item in items if item["client_id"] == _client_id]
            # response = table.query(
            #     KeyConditionExpression=boto3.dynamodb.conditions.Key('client_id').eq(_client_id)
            # )
            # available_profile_names = [profile["profile_name"] for profile in response]

        except Exception as e:
            logger.error(f"Error getting available profile names: {e}")
            available_profile_names = []

        return available_profile_names


def update_available_profile_names(client_id):
    if not client_id:
        return gr.Dropdown(choices=[DEFAULT_PROFILE_NAME], value=DEFAULT_PROFILE_NAME)

    available_profile_names = get_available_profile_names(client_id)
    # available_profile_names = [profile["profile_name"] for profile in available_profiles]
    try:
        selected_profile = DEFAULT_PROFILE_NAME
    except IndexError:
        default_config = AgentConfig(client_id=client_id, profile_name=DEFAULT_PROFILE_NAME)
        default_config.save()
        selected_profile = DEFAULT_PROFILE_NAME
        available_profile_names = [DEFAULT_PROFILE_NAME]
    return gr.Dropdown(choices=available_profile_names, value=selected_profile)


def get_llm_choices(llm_types=None):
    """Get the available LLM choices based on the selected types"""
    if not llm_types:
        llm_types = ["function"]
    available_llms = []
    known_llm_types = openbrain.orm.model_agent_config.EXECUTOR_MODEL_TYPES
    for llm_type in llm_types:
        if llm_type == "function":
            available_llms += openbrain.orm.model_agent_config.FUNCTION_LANGUAGE_MODELS
        elif llm_type == "chat":
            available_llms += openbrain.orm.model_agent_config.CHAT_LANGUAGE_MODELS
        elif llm_type == "completion":
            available_llms += openbrain.orm.model_agent_config.COMPLETION_LANGUAGE_MODELS
        else:
            logger.error(f"Unknown LLM type: {llm_type}, must be one of {known_llm_types}")
            continue
    return gr.Dropdown(choices=available_llms)


def greet(request: gr.Request):
    try:
        return f"Welcome to Gradio, {request.username}"
    except Exception:
        return "OH NO!"


def initialize_username(request: gr.Request):
    _username = request.username or GUEST_CLIENT_ID
    logger.debug(f"Initializing username: {_username}")
    registered_client_ids = get_registered_client_ids()
    # Copying the dict so we don't update it
    registered_client_ids_copy = registered_client_ids.copy()
    registered_client_ids_copy.append(_username)
    client_id_dropdown = gr.Dropdown(choices=registered_client_ids_copy, value=_username)

    try:
        AgentConfig.get(client_id=_username, profile_name=DEFAULT_PROFILE_NAME)
    except Exception as e:
        logger.info(f"No default AgentConfig found for {_username}, creating default agent config for {_username}")
        new_default_agent_config = AgentConfig(client_id=_username, profile_name=DEFAULT_PROFILE_NAME)
        new_default_agent_config.save()
    return [_username, client_id_dropdown]
    # return [_username]

def update_client_id(_username, _session_state):
    _username = _session_state["username"]
    _username_from_session_state = _session_state["username"]

    if _username != _username_from_session_state:
        _session_state["username"] = _username
        logger.debug(f"Updating session state (MISMATCH): {_username=}")

    logger.debug(f"Updating client_id: {_username=}")
    logger.debug(f"Updating client_id: {DEFAULT_CLIENT_ID=}")
    registered_client_ids = get_registered_client_ids()
    registered_client_ids_copy = registered_client_ids.copy()
    registered_client_ids_copy.append(_username)

    return gr.Dropdown(choices=registered_client_ids_copy, value=_username)

# def get_session_username(_session_state=None):
#     username = _session_state["username"]
#     _choices = [DEFAULT_CLIENT_ID, username]
#     return gr.Dropdown(choices=_choices, value=username)


def get_help_text() -> str:
    current_dir = pathlib.Path(__file__).parent
    help_text_path = current_dir / "resources" / "help_text.md"
    with open(help_text_path, "r", encoding="utf8") as file:
        # Read line in UTF-8 format
        help_text = file.readlines()
    return ''.join(help_text)

def get_registered_client_ids(customer_name=CUSTOMER_NAME):
    registered_client_ids = REGISTERED_CLIENT_IDS.get(customer_name, [DEFAULT_CLIENT_ID])
    return registered_client_ids








# EXAMPLE_CONTEXT = """
# {
#     "locationId": "LEADMOMENTUMLOCATIONID",
#     "calendarId": "LEADMOMENTUMCALENDARID",
#     "contactId": "CONTACTID",
#     "random_word": "spatula",
#     "firstName": "Cary",
#     "lastName": "Nutzington",
#     "name": "Cary Nutzington",
#     "dateOfBirth": "1970-04-01",
#     "phone": "+16198675309",
#     "email": "example@email.com",
#     "address1": "1234 5th St N",
#     "city": "San Diego",
#     "state": "CA",
#     "country": "US",
#     "postalCode": "92108",
#     "companyName": "Augmenting Integrations",
#     "website": "openbra.in",
#     "medications": "tylonol"
# }
# """.strip()

EXAMPLE_CONTEXT = {
    "locationId": "LEADMOMENTUMLOCATIONID",
    "calendarId": "LEADMOMENTUMCALENDARID",
    "contactId": "CONTACTID",
    "random_word": "spatula",
}
def set_client_id(_username):
    registered_client_ids = get_registered_client_ids()
    # Copying the dict so we don't update it
    registered_client_ids_copy = registered_client_ids.copy()
    registered_client_ids_copy.append(_username)
    client_id = gr.Dropdown(
        label="Client ID",
        filterable=True,
        info="Develop your own AI agents or use a community agent.",
        choices=registered_client_ids_copy,
        value=_username,
        elem_id="client_id",
        elem_classes=["agent_config"],
    )

    return client_id


def fetch_user_from_leadmo(fetch_user_from_leadmo, context, username):
    """Fetch user from Lead Momentum. If that fails, create a new user."""

    if not fetch_user_from_leadmo:
        return context

    time.sleep(3.0)

    user = Client.get(email=username)

    try:
        leadmo_api_key = user.leadmo_api_key
    except Exception as e:
        gr.Error(f"Please save your Lead Momentum Location ID and API key to your profile.")
        return context

    # First try to fetch with the phone number, and if that fails, try with the email address
    LEADMO_API_V1_CONTACTS_URL = 'https://rest.gohighlevel.com/v1/contacts/'
    LEADMO_API_V1_CONTACTS_LOOKUP_URL = f'{LEADMO_API_V1_CONTACTS_URL}lookup'

    url_encoded_phone = user.phone.replace("+", "%2B")
    url_encoded_email = username.replace("@", "%40")

    phone_url = LEADMO_API_V1_CONTACTS_LOOKUP_URL + f'?phone={url_encoded_phone}'
    email_url = LEADMO_API_V1_CONTACTS_LOOKUP_URL + f'?email={url_encoded_email}'
    found = False
    headers = {
        "Content-Type": "application/json",
        "Origin": DEFAULT_ORIGIN,
        "Authorization": f'Bearer {leadmo_api_key}'
    }
    response = requests.get(url=phone_url, headers=headers)
    if not (200 <= response.status_code < 300):
        logger.error(f"Failed to fetch user from Lead Momentum with phone number.")
    else:
        gr.Info(f"Found user in Lead Momentum with phone number.")
        found = True

    if not found:
        response = requests.get(url=email_url, headers=headers)

        if not (200 <= response.status_code < 300):
            logger.error(f"Failed to fetch user from Lead Momentum with email address.")
        else:
            gr.Info(f"Found user in Lead Momentum with email address.")
            found = True

    if not found:
    # Create a contact

        data = {
            "email": username,
            "phone": user.phone,
            "firstName": user.first_name,
            "lastName": user.last_name,
            "name": f"{user.first_name} {user.last_name}",
            "dateOfBirth": user.date_of_birth,
            "address1": user.address1,
            "city": user.city,
            "state": user.state,
            # "country": user.country,
            "postalCode": user.postal_code,
            "companyName": "Augmenting Integrations",
            "website": user.website,
            "tags": ["woxom_ai"]
        }

        response = requests.post(url=LEADMO_API_V1_CONTACTS_URL, headers=headers, data=json.dumps(data))

        if not (200 <= response.status_code < 300):
            logger.error(f"Failed to create user in Lead Momentum.")
            return context
        else:
            gr.Info(f"Created user in Lead Momentum.")


    context = response.json()["contacts"][0]
    context = {
        "locationId": user.leadmo_location_id,
        "calendarId": user.leadmo_calendar_id,
        "contactId": context.get("id", ""),
        "firstName": context.get("firstName", ""),
        "lastName": context.get("lastName", ""),
        "name": context.get("name", ""),
        "dateOfBirth": context.get("dateOfBirth", ""),
        "phone": context.get("phone", ""),
        "email": context.get("email", ""),
        "address1": context.get("address1", ""),
        "city": context.get("city", ""),
        "state": context.get("state", ""),
        "country": context.get("country", ""),
        "postalCode": context.get("postalCode", ""),
        "companyName": context.get("companyName", ""),
        "website": context.get("website", ""),
    }
    return context    # context2, leadmo_contact_id_phone, leadmo_location_id, lls_api_key


# username.change(fill_context, inputs=[context, username], outputs=user_leadmo_details + user_personal_details)
def fill_context(username, leadmo_api_key, leadmo_location_id, leadmo_calendar_id, first_name, last_name, date_of_birth, phone, address1, address2, city, state, country, postal_code, website):
    """Fill the context with the user's details from Lead Momentum"""
    random_word = faker.Faker().word()
    context = {
        "locationId": leadmo_location_id,
        "calendarId": leadmo_calendar_id,
        "contactId": "",
        "firstName": first_name,
        "lastName": last_name,
        "name": f"{first_name} {last_name}",
        "dateOfBirth": date_of_birth,
        "phone": phone,
        "email": username,
        "address1": address1,
        "address2": address2,
        "city": city,
        "state": state,
        "country": country,
        "postalCode": postal_code,
        "website": website,
        "random_word": random_word
    }
    return context

def fill_context_from_username(username):
    try:
        user = Client.get(email=username)
        context = {
            "locationId": user.leadmo_location_id,
            "calendarId": user.leadmo_calendar_id,
            "contactId": "",
            "firstName": user.first_name,
            "lastName": user.last_name,
            "name": f"{user.first_name} {user.last_name}",
            "dateOfBirth": user.date_of_birth,
            "phone": user.phone,
            "email": username,
            "address1": user.address1,
            "address2": user.address2,
            "city": user.city,
            "state": user.state,
            "country": user.country,
            "postalCode": user.postal_code,
            "website": user.website,
            "random_word": faker.Faker().word()
        }
    except Exception as e:
        logger.warning(f"User not found: {username}")
        context = {
            "locationId": "UNKNOWN",
            "email": username,
        }
    return context

# def get_action_events(_events=None, _session_state=None):
#     try:
#         _session_id = _session_state["session_id"]
#         logger.debug(f"Getting Action Events: session_id {_session_id}")
#     except TypeError:
#         logger.debug(f"Getting Action Events: failed to find session_id")
#
#         return json.dumps({"Idle": "Start a conversation to begin monitoring for events"})
#     logger.debug("Getting latest action...")
#     if not _session_id:
#         return json.dumps({"Idle": "Start a conversation to begin monitoring for events"})
#     try:
#         dynamodb = boto3.resource("dynamodb")
#         table = dynamodb.Table(config.ACTION_TABLE_NAME)
#         # response = table.get_item(Key={"action_id": "latest", "session_id": _session_id})
#         response = table.scan(
#             FilterExpression=Attr('session_id').eq(_session_id)
#         )
#         # ret = response.get("Item", {})
#         ret = response.get("Items", {})
#         logger.debug(f"Action Events: {ret}")
#     except KeyError as e:
#         logger.warning(f"Error finding events: {e}")
#         ret = json.dumps({"exception": "Event not found for this session, perhaps one wasn't sent yet in this conversation"})
#     except Exception as e:
#         logger.error(f"Error finding events: {e}")
#         ret = json.dumps({"exception": e.__str__()})
#
#     if _events:
#         _events = ret
#     return json.dumps(ret, cls=CustomJsonEncoder, indent=4, sort_keys=True)


def alert_on_actions(_session_state):
    """Alert on actions"""
    _events = {}
    for i in range(4):
        time.sleep(5)
        logger.debug(f"Checking for actions({i})...")

        try:
            _session_id = _session_state["session_id"]
            logger.debug("Checking for actions: Found session_id")
        except TypeError:
            logger.warning("No session ID found")
            return [_session_state, _events]

        try:
            dynamodb = boto3.resource("dynamodb")
            table = dynamodb.Table(config.ACTION_TABLE_NAME)
            response = table.scan(
                FilterExpression=Attr('session_id').eq(_session_id)
            )
            logger.debug(f"Checking for actions: {_session_id}")

            event_dict: dict = response.get("Items", {})
            logger.debug(f"found ({len(event_dict)}) actions")

            for item in event_dict:
                event = item.get("event", "")
                response = item.get("response", "")
                logger.debug(f"Event: {event}")
                logger.debug(f"Response: {response}")
                if item["action_id"] in _session_state.get("events") or item["action_id"] == 'latest':
                    continue
                logger.info(f"Adding action_id: {item['action_id']} to session state")
                _session_state.get("events").append(item["action_id"])

                if _session_state.get("message_count") == 0:
                    logger.info(f"Message count is 0")
                    # continue
                # pretty_response = json.dumps(eval(response), indent=2)
                gr.Info(f"Tool used: {event}")
            events_json = json.dumps(event_dict[-5:], cls=CustomJsonEncoder, indent=4, sort_keys=True)
            return [_session_state, events_json]

        except Exception as e:
            logger.error(f"Error checking for actions: {e}")
            return [_session_state, _events]

    return [_session_state, _events]

def warn_if_missing_tool_info(username, tools):
    """Warn if leadmo_location_id and leadmo_api_key are missing from the user's saved profile"""
    try:
        user = Client.get(email=username)
    except Exception as e:
        logger.warning(f"user not found: {username}")
        gr.Warning("Please save your Lead Momentum Location ID and API key to your profile to start using Lead Momentum tools.")
        return
    missing_info = []
    for tool in tools:
        if tool.startswith("leadmo_"):
            if not user.leadmo_location_id:
                missing_info.append("Lead Momentum Location ID")
            if not user.leadmo_api_key:
                missing_info.append("Lead Momentum API Key")
            if not user.leadmo_calendar_id:
                missing_info.append("Lead Momentum Calendar ID")
    if missing_info:
        missing_info_str = ", ".join(missing_info)
        gr.Warning(f"Missing info: {missing_info_str}")

def get_bucket_name():
    try:
        infra_stack_name = config.INFRA_STACK_NAME
        # Get tablename from outputs of INFRA_STACK
        cfn = boto3.client("cloudformation")
        response = cfn.describe_stacks(StackName=infra_stack_name)
        outputs = response["Stacks"][0]["Outputs"]
        for output in outputs:
            if output["OutputKey"] == "ObBucketName":
                bucket = output["OutputValue"]
                break
        else:
            raise Exception("Bucket name not found in outputs")
        return bucket
    except Exception as e:
        raise e

def react_to_message(chatbot, session_state, profile_name, client_id, like_data: gr.LikeData):
    bucket_name = get_bucket_name()
    session_id = session_state.get("session_id").lower()
    agent_config = AgentConfig.get(client_id=client_id, profile_name=profile_name)

    item = {
        "agent_config": agent_config.to_dict(),
        "chatbot_value": chatbot,
        "liked_message": like_data.value,
        "liked_index": like_data.index,
        "liked_or_disliked_as_bool": like_data.liked
    }

    s3 = boto3.client("s3")
    s3_key_name = f"reactions/{profile_name}/{client_id}/{session_id}.json"
    try:
        # get the file so we can append to it
        file = s3.get_object(Bucket=bucket_name, Key=s3_key_name)
        # json_content = json.loads(file["Body"].read())
        jsonl_content_str = file["Body"].read().decode('utf-8')
        jsonl_content = json.loads(jsonl_content_str)

    except Exception as e:
        jsonl_content = []

    jsonl_content.append(item)
    try:

        s3.put_object(Bucket=bucket_name, Key=s3_key_name, Body=json.dumps(jsonl_content))
        gr.Info(f"Feedback successfully saved. Thank you!")
    except Exception as e:
        logger.debug(f"Dumping file on local filesystem...")
        gr.Warning(f"Something went wrong, feedback saved to local filesystem. Please inform your administrator. Thank you!")

